/*-
 * Copyright (C)2022..2025 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2022..2025 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: out.c,v 1.54 2025/01/18 23:06:13 babolo Exp $"

#define BLIN_COMPAT      4
#define Bpars_COMPAT     4
#define MULAR_COMPAT     0
#define MIFE_COMPAT      5
#define RECOBE_COMPAT    6
#define PGOBLIN_COMPAT   5
#define PGOBLIN_INTERNAL 1
#define PGOBLIN_MODULE   pgoblin_module

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <babolo/recobe.h>
#include <multilar.h>
#include <mife.h>
#include <pgoblin.h>
#include "pgoblin5json.h"

#define Nq  0x01U            /*  NULL  Q                                 */
#define Nr  0x02U            /*  NULL  R                                 */
#define Nv  0x04U            /*  NULL  V                                 */
#define No  0x08U            /*                                           */
#define Ob  0x10U            /*                                */
#define Oa  (1 * Ob)         /* 0x10 ,                       */
#define Oc  (2 * Ob)         /* 0x20 ,                           */
#define Or  (4 * Ob)         /* 0x40                                       */
#define Oos (0 * Ob)         /* 0x00                              */
#define Ooa (Oa)             /* 0x10                                */
#define Ocs (Oc)             /* 0x20                               */
#define Oca (Oc | Oa)        /* 0x30                                 */

#define Cc 0x000100U   /*                         */
#define Cs 0x000200U   /*  ,           */
#define Cz 0x000400U   /*  ,                            */
#define Cn 0x000800U   /*                               */
#define Co 0x001000U   /*                         */
#define Ct 0x002000U   /*                                    */
#define C0 0x004000U   /*  {                                          */
#define C1 0x008000U   /*  [                                          */
#define C2 0x010000U   /*  }                                          */
#define C3 0x020000U   /*  ]                                          */
#define C4 0x040000U   /*                                      */

static const char spaces[] = "                                                                ";

typedef struct stapair {
    u_int32_t   cxtflg;
#   define            STYLE_JSON_BOOL     0x000010U  /*   bool                             */
#   define            STYLE_JSON_TINT     0x000020U  /*                               */
#   define            STYLE_JSON_TEXT     0x000030U  /*   text                             */
#   define            STYLE_JSON_TYPE   ( STYLE_JSON_BOOL | STYLE_JSON_TINT | STYLE_JSON_TEXT )
#   define            STYLE_JSON_SQOND    0x000100U  /*                         */
#   define            STYLE_JSON_SRREF    0x001000U  /* @                                            */
#   define            STYLE_JSON_CNTAR    0x002000U  /*  A  S                           */
#   define            STYLE_JSON_SRQUO    0x008000U  /*                                     */
#   define            STYLE_JSON_NONEN    0x400000U  /*                                       */
#   define            STYLE_JSON_CNTOM    0x800000U  /*     level > 0             */
#if BLINAP > 4
    int32_t     dummy;
#endif
    size_t      oln;                                 /*                          */
    const char *onm;                                 /*                                 */
} stapair;

static const u_int32_t automa[5 * Ob] =
{    Cs|Cn|Co|   C0  /* 00 Oos */
,          Co        /* 01 */
,    Cs|Cn|Co|   C0  /* 02 */
,          Co        /* 03 */
,    Cs|Cn|Co|   C0  /* 04 */
,          Co        /* 05 */
,    Cs|Cn|Co|   C0  /* 06 */
,          Co        /* 07 */
,          Co        /* 08 */
,          Co        /* 09 */
,          Co        /* 0A */
,          Co        /* 0B */
,          Co        /* 0C */
,          Co        /* 0D */
,          Co        /* 0E */
,          Co        /* 0F */
,    Cs|Cn|Co|   C1  /* 10 Ooa */
,          Co        /* 11 */
,    Cs|Cn|Co|   C1  /* 12 */
,          Co        /* 13 */
,    Cs|Cn|Co|   C1  /* 14 */
,          Co        /* 15 */
,    Cs|Cn|Co|   C1  /* 16 */
,          Co        /* 17 */
,          Co        /* 18 */
,          Co        /* 19 */
,          Co        /* 1A */
,          Co        /* 1B */
,          Co        /* 1C */
,          Co        /* 1D */
,          Co        /* 1E */
,          Co        /* 1F */
, Cc|Cz|         C2  /* 20 Ocs */
, Cc                 /* 21 */
, Cc|Cz|         C2  /* 22 */
, Cc                 /* 23 */
, Cc|Cz|         C2  /* 24 */
, Cc                 /* 25 */
, Cc|Cz|         C2  /* 26 */
, Cc                 /* 27 */
, Cc                 /* 28 */
, Cc                 /* 29 */
, Cc                 /* 2A */
, Cc                 /* 2B */
, Cc                 /* 2C */
, Cc                 /* 2D */
, Cc                 /* 2E */
, Cc                 /* 2F */
, Cc|Cz|         C3  /* 30 Oca */
, Cc                 /* 31 */
, Cc|Cz|         C3  /* 32 */
, Cc                 /* 33 */
, Cc|Cz|         C3  /* 34 */
, Cc                 /* 35 */
, Cc|Cz|         C3  /* 36 */
, Cc                 /* 37 */
, Cc                 /* 38 */
, Cc                 /* 39 */
, Cc                 /* 3A */
, Cc                 /* 3B */
, Cc                 /* 3C */
, Cc                 /* 3D */
, Cc                 /* 3E */
, Cc                 /* 3F */
,    Cs|Cn|   Ct|C4  /* 40 Or */
,                 0  /* 41 */
,    Cs|Cn|   Ct|C4  /* 42 */
,                 0  /* 43 */
,    Cs|Cn|   Ct|C4  /* 44 */
,                 0  /* 45 */
,    Cs|Cn|   Ct|C4  /* 46 */
,                 0  /* 47 */
,                 0  /* 48 */
,                 0  /* 49 */
,                 0  /* 4A */
,                 0  /* 4B */
,                 0  /* 4C */
,                 0  /* 4D */
,                 0  /* 4E */
,                 0  /* 4F */
};

static int
/*****************************************************************************************************
 ****                                                                      ****
 *****************************************************************************************************
 ****                                                                                             ****/
wr(pgoblin_realsty *st, pgoblin_rio *rou, void const *what, size_t howmach) {                    /****
 ****                                                                                             ****
 *****************************************************************************************************/
#   define blin_internal_flags (st->flags & BLIN_MASK)
    ssize_t e = 0;

    ifBLIN_QX4( "+ %08X %"BLIN_X"=%.*s~ %08X %d"
              , st->flags
              , BLIN_I(rou)
              , (int)howmach, what
              , rou->flags
              , rou->eof
              );
    if  (!!st->shift) {
        e = (int)pgoblin_io_write(rou, what, howmach);
    } else if (1 < howmach) {
        e = (int)pgoblin_io_write(rou, what, howmach - 1);
    }
    e = (0 > e) ? (int)e : 0;
    ifBLIN_QX4("- %zd", e);
    return((int)e);
#   undef blin_internal_flags
}

static u_int32_t
/*****************************************************************************************************
 ****                                                                             ****
 *****************************************************************************************************
 ****                                                                                             ****/
nametonum(pgoblin_exenv *exenv, pgoblin_nr nin, char *text, int coded) {                         /****
 ****                                                                                             ****
 *****************************************************************************************************/
#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
    pgoblin_rio  *rin;
    u_int32_t     ex = 0xFFFFFFFF;
    char         *t  = NULL;
    ssize_t       n  ;

    GET_RIO(rin, exenv->options, nin);
    ifBLIN_QX3("+ %d %"BLIN_X"=%s~", coded, BLIN_I(text), text);
    if  (!coded) {
        t = text;
    } else if (!(t = strdup(text))) {
        ifBLIN_QW0("strdup");
        goto out;
    } else {
        for (int i = 0, j = 0; !!t[i]; ) {
            switch(t[i]) {
            case 0   :
                t[j++] = 0;
                break;
            case '\\':
                switch(t[++i]) {
                case '"' : t[j++] = '"' ; break;
                case '\\': t[j++] = '\\'; break;
                case 'b' : t[j++] = '\b'; break;
                case 'f' : t[j++] = '\f'; break;
                case 'n' : t[j++] = '\n'; break;
                case 'r' : t[j++] = '\r'; break;
                case 't' : t[j++] = '\t'; break;
                default  : t[j++] = '\\';
                           t[j++] = t[i]; break;
                }
                ++i;
                break;
            default  :
                t[j++] = t[i++];
    }   }   }
    n = (ssize_t)pgoblin_db_resinfo(rin, PGOBLIN_Nfields);
    for (ssize_t k = 0; k < n; ++k) {
        const char *reanm = pgoblin_db_subinfo(rin, PGOBLIN_ColName, k);

        if  (!!reanm && !strcmp(t, reanm)) {
            ex = (u_int32_t)k;
            goto out;
    }   }
out:
    if  (!!coded) free(t);
    ifBLIN_QX3("- %u", ex);
    return(ex);
#   undef blin_internal_flags
}

static int
/*****************************************************************************************************
 ****                                        ****
 *****************************************************************************************************
 ****                                                                                             ****/
doshift(pgoblin_rio *rou, int level, u_int8_t shift, int comma) {                                /****
 ****                                                                                             ****
 *****************************************************************************************************/
    int ex = EX_OK;

    if  (!comma) {
        if  (!!shift) for (int i = 0; i < level; ++i) pgoblin_io_write(rou, spaces, shift);
    } else {
        if  ((1 < level) && !!shift) {
            for (int i = 1; i < level; ++i) pgoblin_io_write(rou, spaces, shift);
        }
        if  (!!level) {
            if  (1 < shift) {
                if  (0 > pgoblin_io_write(rou, ",", 1)) {
                    ifBLIN_QW0("pgoblin_io_write");
                    ex = -1;
                    goto out;
                }
                if  (0 > pgoblin_io_write(rou, spaces, shift - 1)) {
                    ifBLIN_QW0("pgoblin_io_write");
                    ex = -1;
                    goto out;
                }
            } else {
                if  (0 > pgoblin_io_write(rou, ", ", 2)) {
                    ifBLIN_QW0("pgoblin_io_write");
                    ex = -1;
                    goto out;
    }   }   }   }
out:
    return(ex);
}

static int
/*****************************************************************************************************
 ****                                                                 ****
 *****************************************************************************************************
 ****                                                                                             ****/
recout(pgoblin_rio *rou, pgoblin_realsty *st, const char *text, ssize_t length) {                /****
 ****                                                                                             ****
 *****************************************************************************************************/
#   define blin_internal_flags (st->flags & BLIN_MASK)
    recobe_chunk *tout = NULL;
    int           ex   = 0;

    ifBLIN_QX3("+ %08X %08X |%.*s~", rou->flags, st->flags, (int)length, text);
    if  (!!st->chain0) {
        /* const char *text   recobe_chainarr(..., void *) *
         *  mife_ctlbuff(..., void *, ...),                              *
         *   mife_descriptor-> void *buffer               *
         *    mife  .                         *
         * recobe      ,                    *
         *      .                                    *
         * ,   ,  .              XXXX */
        if  (!(tout = recobe_chainarr(st->chain, (size_t)length, text))) {
            ifBLIN_QW0("recobe_chainarr |%.*s~", (int)length, text);
            ex = -EX_SOFTWARE;
            goto out;
        }
        text = (const char *)recobe_chunktext(tout);
        length = (ssize_t)recobe_chunksize(tout);
    }
    if  (0 > (ex = (int)pgoblin_io_write(rou, text, (size_t)length))) {
        ifBLIN_QW0("write |%.*s~", (int)length, text);
        ex = -EX_IOERR;
        goto out;
    } else if (ex != (int)length) {
        ifBLIN_QW0("write %d <> %d", ex, (int)length);
        ex = -EX_PROTOCOL;
        goto out;
    }
    free(tout);
    tout = NULL;
out:
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

static int
/*****************************************************************************************************
 ****                                                                  ****
 *****************************************************************************************************
 ****                                                                                             ****/
dolit(pgoblin_rio *rou, pgoblin_realsty *st, const char *text, ssize_t length, int q) {          /****
 ****                                                                                             ****
 *****************************************************************************************************/
    int     ex = 0;

    if  (!q) pgoblin_io_write(rou, "\"", 1);
    ex = recout(rou, st, text, length);
    if  (!q) pgoblin_io_write(rou, "\"", 1);
    if  ((!q) && (0 >= 0)) ex += 2;
    return(ex);
}

static int
/*****************************************************************************************************
 ****                                                                   ****
 *****************************************************************************************************
 ****                                                                                             ****/
doid(pgoblin_rio *rou, pgoblin_realsty *st, const char *text, ssize_t length) {                  /****
 ****                                                                                             ****
 *****************************************************************************************************/
    char   *totx;
    int     p  = 0;

    if  (!(totx = malloc((size_t)length * 2 + 3))) {
        ifBLIN_QW0("totx");
        p = -EX_OSERR;
        goto out;
    }
    totx[p++] = '\"';
    for (ssize_t i = 0; i < length; ++i) {
        switch(text[i]) {
        case '"' : totx[p++] = '\\'; totx[p++] = '\"'; break;
        case '\\': totx[p++] = '\\'; totx[p++] = '\\'; break;
        case '\b': totx[p++] = '\\'; totx[p++] = 'b';  break;
        case '\f': totx[p++] = '\\'; totx[p++] = 'f';  break;
        case '\n': totx[p++] = '\\'; totx[p++] = 'n';  break;
        case '\r': totx[p++] = '\\'; totx[p++] = 'r';  break;
        case '\t': totx[p++] = '\\'; totx[p++] = 't';  break;
        default  : totx[p++] = text[i];                break;
    }   }
    totx[p++] = '\"';
    totx[p] = '\0';
    p = recout(rou, st, totx, p);
out:
    return(p);
}

static int
/*****************************************************************************************************
 ****                                      ****
 *****************************************************************************************************
 ****                                                                                             ****/
recond(pgoblin_rio *rou, pgoblin_realsty *st, stapair *cxt, int level) {                         /****
 ****                                                                                             ****
 *****************************************************************************************************/
#   define blin_internal_flags (st->flags & BLIN_MASK)
    int ex = EX_OK;
    int i;

    ifBLIN_QX3("+ %08X %d=%.*s~", cxt[level].cxtflg, level, (int)cxt[level].oln, cxt[level].onm);
    for (i = level; (i >= 0) && (STYLE_JSON_SQOND & cxt[i].cxtflg); --i) {
        ifBLIN_QX3(" %08X %d", cxt[i].cxtflg, i);
    }
    ++i;
    for (; i <= level; ++i) {
        cxt[i].cxtflg &= ~STYLE_JSON_SQOND;
        if  (0 > (ex = doshift(rou, i - 1, st->shift, STYLE_JSON_CNTOM & cxt[i - 1].cxtflg))) {
            ifBLIN_QW0("doshift");
            goto out;
        }
        if  (1 < i) {
            if  (!(STYLE_JSON_CNTAR & cxt[i - 1].cxtflg)) {
                if  (STYLE_JSON_SRREF & cxt[i].cxtflg) {
                    if  (0 > (ex = doid(rou, st, cxt[i].onm, (ssize_t)cxt[i].oln))) {
                        ifBLIN_QW0("doid");
                        goto out;
                    }
                } else {
                    ex = dolit( rou
                              , st
                              , cxt[i].onm
                              , (ssize_t)cxt[i].oln
                              , STYLE_JSON_SRQUO & cxt[i].cxtflg
                              );
                    if  (0 > ex) {
                        ifBLIN_QW0("dolit");
                        goto out;
                }   }
                pgoblin_io_write(rou, " : ", 3);
            }
            if  (STYLE_JSON_CNTAR & cxt[i].cxtflg) {
                if  (0 > (ex = wr(st, rou, "[\n", 2))) {
                    ifBLIN_QW0("wr");
                    goto out;
                }
            } else {
                if  (0 > (ex = wr(st, rou, "{\n", 2))) {
                    ifBLIN_QW0("wr");
                    goto out;
            }   }
            if  (0 < level) cxt[level - 1].cxtflg |= STYLE_JSON_CNTOM;
        } else if (STYLE_JSON_CNTAR & cxt[i].cxtflg) {
            if  (0 > (ex = wr(st, rou, "[\n", 2))) {
                ifBLIN_QW0("wr");
                goto out;
            }
        } else {
            if  (0 > (ex = wr(st, rou, "{\n", 2))) {
                ifBLIN_QW0("wr");
                goto out;
        }   }
        ifBLIN_QX3(" %08X %d", cxt[i].cxtflg, i);
    }
out:
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

ssize_t
/*****************************************************************************************************
 *****************************************************************************************************
 ****                                                                                             ****/
style_json_table(pgoblin_exenv *exenv, pgoblin_nr *rn, ssize_t low, ssize_t hig) {               /****
 ****                                                                                             ****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
    ssize_t            textlength = 0; /*                                    */
    u_int32_t          control    ;
    BLIN_flag          trapdef    = 0;
    style_json_clause *hlause     ;
    int                level      ;     /*                                             */
    ssize_t            ntyp       ;
    u_int32_t          cond       ;
    pgoblin_rio       *rou        = NULL;
    pgoblin_rio       *rin        ;
    pgoblin_rst       *rst        ;
    ssize_t            row        ;     /*     pq                          */
    stapair           *cxt        ;     /*                                        */
    ssize_t            ex         = -EX_DATAERR;
    pgoblin_realsty   *st         ;

    GET_STY(rst, exenv->options, rn[PGO_CSTY]);
    GET_RIO(rou, exenv->options, rn[PGO_COUT]);
    GET_RIO(rin, exenv->options, rn[PGO_CIN]);
    ex = 0;
    ifBLIN_QX3( "+ %"BLIN_X" %"BLIN_X" %"BLIN_X" [%"BLIN_D",%"BLIN_D"]"
              , BLIN_I(rou)
              , BLIN_I(rst)
              , BLIN_I(rin)
              , low
              , hig
              );
    st = rst->style;
    if  (!!strncmp(st->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
        ifBLIN_QX0("Illegal style %s @STY %c", st->id, pgoblin_regn[rn[PGO_CSTY]]);
        ERROUT(-EX_DATAERR, EINVAL);
    }
    if  (0 >= (ntyp = (ssize_t)pgoblin_db_resinfo(rin, PGOBLIN_Ntuples))) goto out;
    if  (!(hlause = malloc(MULAR_NEXT(st->clause) * sizeof(style_json_clause)))) {
        ifBLIN_QW0("hlause @STY %c", pgoblin_regn[rn[PGO_CSTY]]);
        ex = -EX_OSERR;
        goto out;
    }
    level = st->level;
    trapdef = ~rou->trapdef;
    MARK_IO_OUT_GO(rou);;
    ;;  rou->trapdef = 1;
    MARK_IO_OUT_WENT(rou);;
    for (u_int32_t i = 0; i < MULAR_NEXT(st->clause); ++i) {
        bcopy(mular_getix(st->clause, i), &hlause[i], sizeof(style_json_clause));
        switch(STYLE_JSON_MMSK & hlause[i].flause) {
        case STYLE_JSON_MARR:
            ++level;
            if  (i == st->loop) st->loovel = (u_int16_t)level;
            break;
        case STYLE_JSON_MSTR:
            ++level;
            break;
        case STYLE_JSON_CARR:
            if  ((st->loovel == level) && !st->looe) st->looe = i;
            BLIN_FALLTHROUGH;
        case STYLE_JSON_CSTR:
            --level;
            break;
    }   }
    if  ((st->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
        ifBLIN_QX0( "Illegal style size %d @STY %c"
                  , st->flags & PGOBLIN_NTBSZMK
                  , pgoblin_regn[rn[PGO_CSTY]]
                  );
        ERROUT(-EX_DATAERR, EINVAL);
    }
    if  (!(cxt = calloc(st->lemax + 1, sizeof(stapair)))) {
        ifBLIN_QW0("cxtflg @STY %c", pgoblin_regn[rn[PGO_CSTY]]);
        ex = -EX_OSERR;
        goto out;
    }
    row = 0;
    level = st->level;
    cxt[level].cxtflg = (STYLE_JSON_CNTAR | STYLE_JSON_CNTOM) & st->flags;
    ifBLIN_QX2("%d %d %d %d %d", level, st->lemax, st->loop, st->looe, st->loovel);
    for (u_int i = 0; i < MULAR_NEXT(st->clause); ) {
        ifBLIN_QX4(". %08X %d=%.*s~", cxt[level].cxtflg, level, (int)cxt[level].oln, cxt[level].onm);
        if  (!(STYLE_JSON_DONE & hlause[i].flause)) {
            /*      Q, R, V */
        ;   if  (STYLE_JSON_SQEXT & hlause[i].flause) {
        ;       hlause[i].qnum = nametonum( exenv
                                          , rn[PGO_CIN]
                                          , hlause[i].qtext
                                          , STYLE_JSON_SQQUO & hlause[i].flause
                                          );
        ;       hlause[i].flause |= STYLE_JSON_SQNUM;
        ;   }
        ;   if  (!(STYLE_JSON_MMSK & hlause[i].flause) || (STYLE_JSON_SREXT & hlause[i].flause)) {
        ;       if  (STYLE_JSON_SRNUM & hlause[i].flause) {
        ;           hlause[i].flause &= ~STYLE_JSON_SREXT;
        ;           if  (!(STYLE_JSON_SRREF & hlause[i].flause)) {
        ;               hlause[i].rname = pgoblin_db_subinfo(rin, PGOBLIN_ColName, hlause[i].rnum);
        ;              if  (!!hlause[i].rname) hlause[i].flause |= STYLE_JSON_SREXT;
        ;           } else if (!pgoblin_db_valinfo( rin
                                                  , PGOBLIN_IsNull
                                                  , row
                                                  , (ssize_t)hlause[i].rnum
                              )                   ) {
        ;               hlause[i].rname = pgoblin_db_getvalue(rin, row, (ssize_t)hlause[i].rnum);
        ;               if  (!!hlause[i].rname) hlause[i].flause |= STYLE_JSON_SREXT;
        ;           }
        ;       } else if (STYLE_JSON_SREXT & hlause[i].flause) {
        ;           hlause[i].rnum = nametonum( exenv
                                              , rn[PGO_CIN]
                                              , hlause[i].rtext
                                              , STYLE_JSON_SRQUO & hlause[i].flause
                                              );
        ;           hlause[i].flause |= STYLE_JSON_SRNUM;
        ;   }   }
        ;   if  ((STYLE_JSON_SVREF & hlause[i].flause) && (STYLE_JSON_SVEXT & hlause[i].flause)) {
        ;       hlause[i].vnum = nametonum( exenv
                                          , rn[PGO_CIN]
                                          , hlause[i].vtext
                                          , STYLE_JSON_SQUOT & hlause[i].flause
                                          );
        ;       hlause[i].flause |= STYLE_JSON_SVNUM;
        ;   }
            hlause[i].flause |= STYLE_JSON_DONE;
        }
        /*     */
        switch(STYLE_JSON_MMSK & hlause[i].flause) {
        case 0              : cond = Or ; break;
        case STYLE_JSON_MARR: cond = Ooa; break;
        case STYLE_JSON_MSTR: cond = Oos; break;
        case STYLE_JSON_CARR: cond = Oca; break;
        case STYLE_JSON_CSTR: cond = Ocs; break;
        default             :
            ifBLIN_QX0( "Illegal mode %u @STY %c"
                      , STYLE_JSON_MMSK & hlause[i].flause
                      , pgoblin_regn[rn[PGO_CSTY]]
                      );
            ERROUT(EX_DATAERR, EINVAL);
        }
        /*   NULL  Q, R, V  */
        if  (  (!(STYLE_JSON_MMSK & hlause[i].flause)
               || (STYLE_JSON_SREXT & hlause[i].flause)
               )
            && (  !(STYLE_JSON_SRNUM & hlause[i].flause)
               || (0xFFFFFFFF == hlause[i].rnum)
               || !!pgoblin_db_valinfo(rin, PGOBLIN_IsNull, row, (ssize_t)hlause[i].rnum)
            )  ) cond |= Nr;
        if  (  (STYLE_JSON_SVREF & hlause[i].flause)
            && (  !(STYLE_JSON_SVNUM & hlause[i].flause)
               || (0xFFFFFFFF == hlause[i].vnum)
               || !!pgoblin_db_valinfo(rin, PGOBLIN_IsNull, row, (ssize_t)hlause[i].vnum)
            )  ) cond |= Nv;
        if  (STYLE_JSON_SQOND & hlause[i].flause) {
            if  ((STYLE_JSON_SQEXT | STYLE_JSON_SQNUM) & hlause[i].flause) {
        ;       if  (  !(STYLE_JSON_SQNUM & hlause[i].flause)
                    || (0xFFFFFFFF == hlause[i].qnum)
                    || !!pgoblin_db_valinfo(rin, PGOBLIN_IsNull, row, (ssize_t)hlause[i].qnum)
                    ) cond |= Nq;
        ;   } else if (((STYLE_JSON_MSTR | STYLE_JSON_MARR) & hlause[i].flause) || (Nv & cond)) {
                cond |= Nq;
        }   }
        if  ((STYLE_JSON_MCLO & hlause[i].flause) && (STYLE_JSON_SQOND & cxt[level].cxtflg)) {
            cond |= Nq;
        }
        if  (STYLE_JSON_NONEN & cxt[level].cxtflg) cond |= No;
        control = automa[cond];
        ifBLIN_QX2( "%u %02X %08X %08X %d %"BLIN_X"q%s~ %d %"BLIN_X"r%s~ %d %"BLIN_X
                    "v%s~ %08X[%d]%08X=%.*s~"
                  , (u_int32_t)i
                  , cond
                  , control
                  , hlause[i].flause
                  , hlause[i].qnum
                  , BLIN_I(hlause[i].qtext)
                  , (STYLE_JSON_SQEXT & hlause[i].flause) ? hlause[i].qtext : ""
                  , hlause[i].rnum
                  , BLIN_I(hlause[i].rtext)
                  , (STYLE_JSON_SREXT & hlause[i].flause) ? hlause[i].rtext : ""
                  , hlause[i].vnum
                  , BLIN_I(hlause[i].vtext)
                  , (STYLE_JSON_SVEXT & hlause[i].flause) ? hlause[i].vtext : ""
                  , (0 < level) ? cxt[level - 1].cxtflg : 0
                  , level
                  , cxt[level].cxtflg
                  , (int)cxt[level].oln, cxt[level].onm
                  );
        if  (Cc & control) {
            if  (i == st->looe) {
        ;       if  (-1 == hig) break;
        ;       if  ((ntyp + low > row + 1) || ((0 < hig) && (row + 1 < hig))) {
        ;           ++row;
        ;           i = st->loop + 1;
        ;           continue;
        ;   }   }
            --level;
        }
        if  (Cs & control) {
            if  (0 > (ex = recond(rou, st, cxt, level))) {
        ;       ifBLIN_QW0("recond");
        ;       goto out;
        ;   }
        ;   if  (0 > (ex = doshift(rou, level, st->shift, STYLE_JSON_CNTOM & cxt[level].cxtflg))) {
        ;       ifBLIN_QW0("doshift");
        ;       goto out;
        ;   }
            cxt[level].cxtflg |= STYLE_JSON_CNTOM;
        }
        if  ((Cz & control) && (0 > (ex = doshift(rou, level, st->shift, 0)))) {
            ifBLIN_QW0("doshift");
            goto out;
        }
        if  ((Cn & control) && !!level && !(STYLE_JSON_CNTAR & cxt[level].cxtflg)) {
            if  (STYLE_JSON_SRREF & hlause[i].flause) {
        ;       if  (!(Nr & cond)) {
        ;           ex = doid( rou
                             , st
                             , pgoblin_db_getvalue(rin, row, (ssize_t)hlause[i].rnum)
                             , pgoblin_db_valinfo(rin, PGOBLIN_Length, row, (ssize_t)hlause[i].rnum)
                             );
        ;           if  (0 > ex) {
        ;               ifBLIN_QW0("doid");
        ;               goto out;
        ;           }
        ;           pgoblin_io_write(rou, " : ", 3);
        ;       } else {
        ;           pgoblin_io_write(rou, "\"\" : ", 5);
        ;       }
        ;   } else if (STYLE_JSON_SREXT & hlause[i].flause) {
        ;       ex = dolit( rou
                          , st
                          , hlause[i].rtext
                          , (ssize_t)strlen(hlause[i].rtext)
                          , STYLE_JSON_SRQUO & hlause[i].flause
                          );
        ;       if  (0 > ex) {
        ;           ifBLIN_QW0("dolit");
        ;           goto out;
        ;       }
        ;       pgoblin_io_write(rou, " : ", 3);
        ;   } else {
                pgoblin_io_write(rou, "\"\" : ", 5);
        }   }
        if  (Co & control) {
            cxt[level + 1].cxtflg = (STYLE_JSON_SQOND | STYLE_JSON_TYPE) & cxt[level].cxtflg;
        ;   ifBLIN_QX4( ", %08X %d=%.*s~"
                      , cxt[level].cxtflg
                      , level
                      , (int)cxt[level].oln, cxt[level].onm
                      );
        ;   ++level;
        ;   cxt[level].oln = 0;
        ;   cxt[level].onm = NULL;
        ;   if  (!(STYLE_JSON_CNTAR & cxt[level].cxtflg)) {
        ;       if  (!(STYLE_JSON_SRREF & hlause[i].flause)) {
        ;           cxt[level].onm = hlause[i].rtext;
        ;           cxt[level].cxtflg |= STYLE_JSON_SRQUO & hlause[i].flause;
        ;           if  (!!cxt[level].onm) cxt[level].oln = strlen(cxt[level].onm);
        ;       } else if (!(Nr & cond)) {
        ;           cxt[level].onm = pgoblin_db_getvalue(rin, row, (ssize_t)hlause[i].rnum);
        ;           cxt[level].cxtflg |= STYLE_JSON_SRREF;
        ;           cxt[level].oln = (size_t)pgoblin_db_valinfo( rin
                                                               , PGOBLIN_Length
                                                               , row
                                                               , (ssize_t)hlause[i].rnum
                                                               );
        ;   }   }
        ;   if  (STYLE_JSON_TYPE & hlause[i].flause) {
        ;       cxt[level].cxtflg &= ~STYLE_JSON_TYPE;
        ;       cxt[level].cxtflg |= STYLE_JSON_TYPE & hlause[i].flause;
        ;   }
        ;   if  (Oa & cond) cxt[level].cxtflg |= STYLE_JSON_CNTAR;
        ;   if  (!(STYLE_JSON_SQOND & hlause[i].flause)) {
        ;   } else if (!(STYLE_JSON_SQNUM & hlause[i].flause)) {
        ;       cxt[level].cxtflg |= STYLE_JSON_SQOND;
        ;   } else if (Nq & cond) {
        ;       cxt[level].cxtflg |= STYLE_JSON_NONEN;
        ;   }
            ifBLIN_QX4( ": %08X %d=%.*s~"
                      , cxt[level].cxtflg
                      , level
                      , (int)cxt[level].oln, cxt[level].onm
                      );
        }
        if  ((Ct & control) && !(STYLE_JSON_TYPE & hlause[i].flause)) {
            const char *ct = pgoblin_db_subinfo(rin, PGOBLIN_TypeNm, hlause[i].vnum);

        ;   if  (!!ct) {
        ;       if  (!strncasecmp(ct, "bool", 4)) {
        ;           hlause[i].flause |= STYLE_JSON_BOOL << STYLE_JSON_VTSH;
        ;       } else if (!strncasecmp(ct, "int", 3)) {
        ;           hlause[i].flause |= STYLE_JSON_TINT << STYLE_JSON_VTSH;
        ;       } else if (!strncasecmp(ct, "num", 3)) {
        ;           hlause[i].flause |= STYLE_JSON_TINT << STYLE_JSON_VTSH;
        ;       } else if (!strncasecmp(ct, "float", 5)) {
        ;           hlause[i].flause |= STYLE_JSON_TINT << STYLE_JSON_VTSH;
        ;       } else if (!strncasecmp(ct, "double", 6)) {
        ;           hlause[i].flause |= STYLE_JSON_TINT << STYLE_JSON_VTSH;
        ;       } else {
        ;           hlause[i].flause |= STYLE_JSON_TEXT << STYLE_JSON_VTSH;
        ;       }
        ;   } else {
                hlause[i].flause |= STYLE_JSON_TEXT << STYLE_JSON_VTSH;
        }   }
        if  (C0 & control) {
            if  (0 > (ex = wr(st, rou, "{\n", 2))) {
                ifBLIN_QW0("wr");
                goto out;
            }
            if  (0 < level) cxt[level - 1].cxtflg |= STYLE_JSON_CNTOM;
        }
        if  (C1 & control) {
            if  (0 > (ex = wr(st, rou, "[\n", 2))) {
                ifBLIN_QW0("wr");
                goto out;
            }
            if  (0 < level) cxt[level - 1].cxtflg |= STYLE_JSON_CNTOM;
        }
        if  ((C2 & control) && (0 > (ex = wr(st, rou, "}\n", 2)))) {
            ifBLIN_QW0("wr");
            goto out;
        }
        if  ((C3 & control) && (0 > (ex = wr(st, rou, "]\n", 2)))) {
            ifBLIN_QW0("wr");
            goto out;
        }
        if  (C4 & control) {
            int         type = 0;
        ;   ssize_t     ln   = 0;
        ;   char       *cv   = NULL;

        ;   if  (!(Nv & cond)) {
        ;       type = STYLE_JSON_TYPE & hlause[i].flause;                          /*   */
        ;       if  (!type) type = STYLE_JSON_TYPE & cxt[level].cxtflg;            /*   */
        ;       if  (!type) type =  ((STYLE_JSON_BTYPE & st->flags) >> STYLE_JSON_BTYSH);  /*  */
        ;       if  (!type) type = (STYLE_JSON_VTYP & hlause[i].flause) >> STYLE_JSON_VTSH;  /*  */
        ;       if  (!(STYLE_JSON_SVREF & hlause[i].flause)) {
        ;           cv = hlause[i].vtext;
        ;           ln = (ssize_t)strlen(cv);
        ;           if  (!!cv) {
        ;               if  ((STYLE_JSON_TEXT == type) && !(STYLE_JSON_SQUOT & hlause[i].flause)) {
        ;                   if  (0 > (ex = doid(rou, st, cv, ln))) {
        ;                       ifBLIN_QW0("doid");
        ;                       goto out;
        ;                   }
        ;               } else {
        ;                   if  (0 > (ex = recout(rou, st, cv, ln))) {
        ;                       ifBLIN_QW0("recout");
        ;                       goto out;
        ;               }   }
        ;               if  (0 > (ex = wr(st, rou, "\n", 1))) {
        ;                   ifBLIN_QW0("wr");
        ;                   goto out;
        ;           }   }
        ;       } else if (!pgoblin_db_valinfo(rin, PGOBLIN_IsNull, row, (ssize_t)hlause[i].vnum)) {
        ;           cv = pgoblin_db_getvalue(rin, row, (ssize_t)hlause[i].vnum);
        ;           ln = pgoblin_db_valinfo(rin, PGOBLIN_Length, row, (ssize_t)hlause[i].vnum);
        ;           switch(type) {
                    case STYLE_JSON_BOOL:
        ;               if  (('F' == (0xDF & cv[0])) || ((1 == ln) && ('0' == cv[0]))) {
        ;                   if  (0 > (ex = wr(st, rou, "false\n", 6))) {
        ;                       ifBLIN_QW0("wr");
        ;                       goto out;
        ;                   }
        ;               } else {
        ;                   if  (0 > (ex = wr(st, rou, "true\n", 5))) {
        ;                       ifBLIN_QW0("wr");
        ;                       goto out;
        ;               }   }
        ;               break;
                    case STYLE_JSON_TINT:
        ;               if  (0 > (ex = recout(rou, st, cv, ln))) {
        ;                   ifBLIN_QW0("recout");
        ;                   goto out;
        ;               }
        ;               if  (0 > (ex = wr(st, rou, "\n", 1))) {
        ;                   ifBLIN_QW0("wr");
        ;                   goto out;
        ;               }
        ;               break;
                    default             :
        ;               if  (0 > (ex = doid(rou, st, cv, ln))) {
        ;                   ifBLIN_QW0("doid");
        ;                   goto out;
        ;               }
        ;               if  (0 > (ex = wr(st, rou, "\n", 1))) {
        ;                   ifBLIN_QW0("wr");
        ;                   goto out;
        ;               }
        ;               break;
        ;   }   }   }
        ;   if  (!cv && (0 > (ex = wr(st, rou, "null\n", 5)))) {
        ;       ifBLIN_QW0("wr");
        ;       goto out;
        }   }
        ++i;
    }
out:
    if  (!!rou) {
        MARK_IO_OUT_GO(rou);;
        ;;  rou->trapdef = ~trapdef;
        MARK_IO_OUT_WENT(rou);;
    }
    ifBLIN_QX3("- %zd %"BLIN_D, ex, textlength);
    return((ex < 0) ? ex : textlength);
#   undef blin_internal_flags
}
