/*-
 * Copyright (C)2022..2025 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2022..2025 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: pgoblin5json.c,v 1.28 2025/01/25 00:12:13 babolo Exp $"

#define BLIN_COMPAT      4
#define Bpars_COMPAT     4
#define MULAR_COMPAT     0
#define MIFE_COMPAT      5
#define RECOBE_COMPAT    6
#define PGOBLIN_COMPAT   5
#define PGOBLIN_INTERNAL 1
#define PGOBLIN_MODULE   pgoblin_module

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <babolo/recobe.h>
#include <multilar.h>
#include <mife.h>
#include <pgoblin.h>
#include "pgoblin5json.h"

static const char spaces[] = "                                                                ";
static const char style_json_typemarks[] = "-.*#";

static int
/*****************************************************************************************************
 **                                                                                                 **/
init(pgoblin_exenv *exenv, u_int slot) {                                                           /**
 **                                                                                                 **
 *****************************************************************************************************/
    (void)exenv;
    (void)slot;
    return(0);
}

static int
/*****************************************************************************************************
 **                                                                                                 **/
fini(pgoblin_exenv *exenv) {                                                                       /**
 **                                                                                                 **
 *****************************************************************************************************/
    (void)exenv;
    return(0);
}

static int
/*****************************************************************************************************
 **                                                                                                 **/
clear(pgoblin_rst *rst) {                                                                          /**
 **                                                                                                 **
 *****************************************************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    int                ex = EX_OK;
    style_json_clause *ct;

    ifBLIN_QX3("+ %"BLIN_X, BLIN_I(rst));
    if  (!!rst) {
        if  (!!rst->style) {
            if  (!!strncmp(rst->style->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
                ifBLIN_QX0("Illegal style %s @STY %c", rst->style->id, pgoblin_regn[rst->nst]);
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  ((rst->style->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
                ifBLIN_QX0( "Illegal style size %d @STY %c"
                          , rst->style->flags & PGOBLIN_NTBSZMK
                          , pgoblin_regn[rst->nst]
                          );
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  (rst->style->clause) {
                for (size_t i = 0; i < MULAR_NEXT(rst->style->clause); ++i) {
                    ct = mular_getix(rst->style->clause, i);
                    if  (STYLE_JSON_SVALL & ct->flause) free(ct->vtext);
                }
                mular_destroy(rst->style->clause);
            }
            if  (rst->style->chain) recobe_close(rst->style->chain);
            free(rst->style);
            rst->style = NULL;
            rst->flags &= ~PGOBLIN_STYLE_TYPE;
    }   }
out:
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

static void
/*****************************************************************************************************
 **                                                                                                 **/
qpr(pgoblin_rst *rst, style_json_clause *ct, int level, FILE *ouf) {                               /**
 **                                                                                                 **
 *****************************************************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    int                n = 0;

    (void)blin_internal_flags;
    if  (STYLE_JSON_SQOND & ct->flause) n += fprintf(ouf, "?");
    if  (STYLE_JSON_SQNUM & ct->flause) n += fprintf(ouf, "%u", ct->qnum);
    if  (STYLE_JSON_SQQUO & ct->flause) {
        n += fprintf(ouf, "\"%s\"", ct->qname);
    } else if (STYLE_JSON_SQEXT & ct->flause) {
        n += fprintf(ouf, "'%s'", ct->qname);
    }
    if  (!level) {
        if  (rst->style->shift > n) {
            fprintf(ouf, "%.*s", rst->style->shift - n, spaces);
        } else {
            fprintf(ouf, " ");
        }
    } else if ((level * rst->style->shift) > n) {
        fprintf(ouf, "%.*s", (level * rst->style->shift) - n, spaces);
        if  (rst->style->shift > 1) {
            if  (STYLE_JSON_MCLO & ct->flause) {
                fprintf(ouf, "%.*s", rst->style->shift, spaces);
            } else {
                fprintf(ouf, ",%.*s", rst->style->shift - 1, spaces);
        }   }
    } else {
        n -= level * rst->style->shift;
        if  (rst->style->shift > (n + 1)) {
            fprintf(ouf, ",%.*s", rst->style->shift - n - 1, spaces);
        } else {
            fprintf(ouf, ",");
    }   }
#   undef blin_internal_flags
}

void
/*****************************************************************************************************
 *****************************************************************************************************
 ****                                                                                             ****/
style_json_dump(pgoblin_rst *rst, FILE *ouf) {                                                   /****
 ****                                                                                             ****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    int                level;
    style_json_clause *ct;
    int                n;

    if  (!rst->style) {
        ifBLIN_QX0("Dump json: no style @STY %c", pgoblin_regn[rst->nst]);
        errno = EINVAL;
        goto out;
    }
    if  ((rst->style->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
        ifBLIN_QX0( "Illegal pgoblin json style size %d @STY %c"
                  , rst->style->flags & PGOBLIN_NTBSZMK
                  , pgoblin_regn[rst->nst]
                  );
        errno = EINVAL;
        goto out;
    }
    level = rst->style->level;
    fprintf(ouf, "+######################################\n");
    fprintf( ouf
           , "%08X %08X ^%u >%u"
           , rst->style->flags
           , rst->style->cf.flags
           , rst->style->level
           , rst->style->shift
           );
    if  (0xFFFFFFFF != rst->style->loop) fprintf(ouf, " loop=%u", rst->style->loop);
    if  (!!rst->style->cf.mod) fprintf(ouf, " mod=%s~", rst->style->cf.mod);
    if  (!!rst->style->cf.bsubst) fprintf(ouf, " b=%s~", rst->style->cf.bsubst);
    if  (!!rst->style->cf.esubst) fprintf(ouf, " e=%s~\n", rst->style->cf.esubst);
    fprintf(ouf, "\n");
    for (size_t i = 0; i < MULAR_NEXT(rst->style->clause); ++i) {
        n = 0;
        ct = mular_getix(rst->style->clause, i);
        if  (10 >= MULAR_NEXT(rst->style->clause)) {
            if  ((i == rst->style->loop) && (STYLE_JSON_TYPE & ct->flause)) {
                fprintf( ouf
                       , "[%c]%08X "
                       , style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]
                       , ct->flause
                       );
            } else {
                fprintf(ouf, "[%u]%08X ", (u_int)i, ct->flause);
            }
        } else if (100 >= MULAR_NEXT(rst->style->clause)) {
            if  (i == rst->style->loop) {
                fprintf( ouf
                       , "[ %c]%08X "
                       , style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]
                       , ct->flause
                       );
            } else {
                fprintf(ouf, "[%2u]%08X ", (u_int)i, ct->flause);
            }
        } else if (1000 >= MULAR_NEXT(rst->style->clause)) {
            if  (i == rst->style->loop) {
                fprintf( ouf
                       , "[  %c]%08X "
                       , style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]
                       , ct->flause
                       );
            } else {
                fprintf(ouf, "[%3u]%08X ", (u_int)i, ct->flause);
            }
        } else {
            if  (i == rst->style->loop) {
                fprintf( ouf
                       , "[   %c]%08X "
                       , style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]
                       , ct->flause
                       );
            } else {
                fprintf(ouf, "[%4u]%08X ", (u_int)i, ct->flause);
        }   }
        switch(STYLE_JSON_MMSK & ct->flause) {
        case 0:
            qpr(rst, ct, level, ouf);
            if  (STYLE_JSON_SRREF & ct->flause) fprintf(ouf, "@");
            if  (STYLE_JSON_SRNUM & ct->flause) {
                fprintf(ouf, "%u", ct->rnum);
            } else if (STYLE_JSON_SREXT & ct->flause) {
                fprintf(ouf, "\"%s\"", ct->rname);
            }
            fprintf(ouf, " ");
            if  (STYLE_JSON_SVREF & ct->flause) {
                fprintf(ouf, "@");
                if  (STYLE_JSON_SVNUM & ct->flause) {
                    fprintf(ouf, "%u", ct->vnum);
                } else if (STYLE_JSON_SVEXT & ct->flause) {
                    fprintf(ouf, "\"%s\"", ct->vname);
                }
            } else if (STYLE_JSON_SVVAL & ct->flause) {
                fprintf(ouf, "%s", ct->vname);
            }
            if  (STYLE_JSON_TYPE & ct->flause) {
                fprintf(ouf, " %c", style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]);
            }
            break;
        case STYLE_JSON_MARR:
            qpr(rst, ct, level, ouf);
            n = 0;
            if  (STYLE_JSON_SRREF & ct->flause) n += fprintf(ouf, "@");
            if  (STYLE_JSON_SRNUM & ct->flause) {
                n += fprintf(ouf, "%u", ct->rnum);
            } else if (STYLE_JSON_SREXT & ct->flause) {
                n += fprintf(ouf, "\"%s\"", ct->rname);
            }
            if  (!!n) fprintf(ouf, " ");
            fprintf(ouf, "[");
            if  (i == rst->style->loop) {
                fprintf(ouf, " %c", style_json_typemarks[(STYLE_JSON_TYPE & ct->flause) >> 4]);
            }
            ++level;
            break;
        case STYLE_JSON_MSTR:
            qpr(rst, ct, level, ouf);
            n = 0;
            if  (STYLE_JSON_SRREF & ct->flause) n += fprintf(ouf, "@");
            if  (STYLE_JSON_SRNUM & ct->flause) {
                n += fprintf(ouf, "%u", ct->rnum);
            } else if (STYLE_JSON_SREXT & ct->flause) {
                n += fprintf(ouf, "\"%s\"", ct->rname);
            }
            if  (!!n) fprintf(ouf, " ");
            fprintf(ouf, "{");
            ++level;
            break;
        case STYLE_JSON_CARR:
            --level;
            qpr(rst, ct, level, ouf);
            fprintf(ouf, "]");
            break;
        case STYLE_JSON_CSTR:
            --level;
            qpr(rst, ct, level, ouf);
            fprintf(ouf, "}");
            break;
        default:
            fprintf(ouf, "!!!! %08X", ct->flause);
        }
        fprintf(ouf, "\n");
    }
    if  (rst->style->level != level) {
        fprintf(ouf, " Not balansed %d\n", level - rst->style->level);
        level = 1;
    }
    fprintf(ouf, "-######################################\n");
    fflush(ouf);
out:
    return;
#   undef blin_internal_flags
}

pgoblin_styles PGOBLIN_MODULE =
{ 0
, "5pGoblin-" VERS
, "json\0" VERS
, 0
, init
, fini
, style_json_parser
, style_json_dump
, clear
, NULL /* get    */
, style_json_table
, NULL /* load   */
, NULL /* store  */
, NULL /* dummy0 */
, NULL /* dummy1 */
};
