/*-
 * Copyright (C)2022..2024 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pgoblin5json.h,v 1.27 2024/03/24 20:30:06 babolo Exp $
 */

#ifndef _PGOBLIN_5_JSON_H_
# define _PGOBLIN_5_JSON_H_

__BEGIN_DECLS

#define PGOBLIN_NTBSIZE  16                   /*       0-65535 */
#define PGOBLIN_STYLE_ID "#pGoblin-json"
#define STYLE_JSON_null  1
#define STYLE_JSON_false 2
#define STYLE_JSON_true  3

typedef struct style_json_term {
    u_int32_t         flterm;
                                                         /* !!!              *
                                                          *        .         *
                                                          *     style_json_char    */
#   define            STYLE_JSON_COND     0x000002U      /* ?                                        */
#   define            STYLE_JSON_REFR     0x000004U      /* @                                        */
#   define            STYLE_JSON_BOOL     0x000010U      /* !!!                               */
#   define            STYLE_JSON_TINT     0x000020U      /* !!!                              */
#   define            STYLE_JSON_TEXT     0x000030U      /* !!!                      */
#   define            STYLE_JSON_TYPE   ( STYLE_JSON_BOOL | STYLE_JSON_TINT | STYLE_JSON_TEXT )
#   define            STYLE_JSON_STRU     0x000040U      /* !!!  - {  },  [  ]   */
#   define            STYLE_JSON_OPEN     0x000080U      /* !!!  - [  {,  }  ]      */
#   define            STYLE_JSON_OBJM     (STYLE_JSON_STRU | STYLE_JSON_OPEN)
#   define            STYLE_JSON_NUMR     0x000100U      /*                         */
#   define            STYLE_JSON_WORD     0x000200U      /*                              */
#   define            STYLE_JSON_MISC     0x000400U      /*  -                           */
#   define            STYLE_JSON_QUOT     0x000800U      /*                       */
#   define            STYLE_JSON_OBJC     0x001000U      /*         */
#   define            STYLE_JSON_MODE   ( STYLE_JSON_NUMR \
                                        | STYLE_JSON_WORD \
                                        | STYLE_JSON_MISC \
                                        | STYLE_JSON_QUOT \
                                        | STYLE_JSON_OBJC \
                                        )
#   define            STYLE_JSON_LOOP     0x002000U      /*                        */
#   define            STYLE_JSON_DONE     0x800000U      /*                                */
    u_int32_t         nuterm;
    char             *txterm;
    u_int32_t         nutext;
#if BLINAP > 4
    int32_t           dummy;
#endif
} style_json_term;

typedef struct style_json_clause {
    u_int32_t         flause;
#   define            STYLE_JSON_MSTR     0x000001U  /* {  } ()                          */
#   define            STYLE_JSON_MARR     0x000002U  /* [  ] ()                             */
#   define            STYLE_JSON_MCLO     0x000004U  /*                   */
#   define            STYLE_JSON_CARR  ( STYLE_JSON_MARR | STYLE_JSON_MCLO )
#   define            STYLE_JSON_CSTR  ( STYLE_JSON_MSTR | STYLE_JSON_MCLO )
#   define            STYLE_JSON_MMSK  ( STYLE_JSON_MSTR | STYLE_JSON_MARR | STYLE_JSON_MCLO )
#   define            STYLE_JSON_BOOL     0x000010U  /*   bool                             */
#   define            STYLE_JSON_TINT     0x000020U  /*                               */
#   define            STYLE_JSON_TEXT     0x000030U  /*   text                             */
#   define            STYLE_JSON_TYPE  ( STYLE_JSON_BOOL | STYLE_JSON_TINT | STYLE_JSON_TEXT )
#   define            STYLE_JSON_VTSH     2
#   define            STYLE_JSON_VTYP  ( STYLE_JSON_TYPE << STYLE_JSON_VTSH)
#   define            STYLE_JSON_SQOND    0x000100U  /*                         */
#   define            STYLE_JSON_SQEXT    0x000200U  /*                               */
#   define            STYLE_JSON_SQNUM    0x000400U  /*                               */
#   define            STYLE_JSON_SQQUO    0x000800U  /*                                */
#   define            STYLE_JSON_SRREF    0x001000U  /* @                                            */
#   define            STYLE_JSON_SREXT    0x002000U  /*                                         */
#   define            STYLE_JSON_SRNUM    0x004000U  /*                                         */
#   define            STYLE_JSON_SRQUO    0x008000U  /*                                     */
#   define            STYLE_JSON_SRMSK ( STYLE_JSON_SRREF | STYLE_JSON_SREXT | STYLE_JSON_SRNUM )
#   define            STYLE_JSON_SVREF    0x010000U  /* @                                            */
#   define            STYLE_JSON_SVEXT    0x020000U  /*                                         */
#   define            STYLE_JSON_SVNUM    0x040000U  /*                                         */
#   define            STYLE_JSON_SQUOT    0x080000U  /*                                     */
#   define            STYLE_JSON_SVVAL    0x100000U  /*                                      */
#   define            STYLE_JSON_SVALL    0x200000U  /*  free()                                  */
#   define            STYLE_JSON_SVMSK ( STYLE_JSON_SVREF \
                                       | STYLE_JSON_SVEXT \
                                       | STYLE_JSON_SVNUM \
                                       | STYLE_JSON_SVVAL \
                                       )             /* !!!     V  R  */
#   define            STYLE_JSON_DONE     0x800000U  /*                             */
    u_int32_t         dummy;
    u_int32_t         dummy1;
    u_int32_t         qnum;
    u_int32_t         rnum;
    u_int32_t         vnum;
    union {
        char             *qtext;
        char             *qname;
    };
    union {
        char             *rtext;
        const char       *rname;
    };
    union {
        char             *vtext;
        char             *vname;
    };
} style_json_clause;

struct pgoblin_realsty {
    char              id[PGOBLIN_STRING_ID_LEN];     /*  "#pGoblin"          */
    BLIN_flag         flags;
#   define            PGOBLIN_NTBSZMK     0x0000FFU  /*  */
#   define            STYLE_JSON_BTYSH    4
#   define            STYLE_JSON_BBOOL  ( STYLE_JSON_BOOL << STYLE_JSON_BTYSH)     /* 0x000100 */
#   define            STYLE_JSON_BTINT  ( STYLE_JSON_TINT << STYLE_JSON_BTYSH)     /* 0x000200 */
#   define            STYLE_JSON_BTEXT  ( STYLE_JSON_TEXT << STYLE_JSON_BTYSH)     /* 0x000300 */
#   define            STYLE_JSON_BTYPE  ( STYLE_JSON_BBOOL | STYLE_JSON_BTINT | STYLE_JSON_BTEXT )
#   define            STYLE_JSON_CNTAR    0x002000U  /*  A  S                           */
#   define            STYLE_JSON_SQUOT    0x080000U  /*                                     */
#   define            STYLE_JSON_CNTOM    0x800000U  /*     level > 0             */
    u_int16_t         level;                         /*           */
    u_int16_t         lemax;                         /*                        */
    mular_descriptor *clause;                        /*                                   */
    recobe_chain     *chain0;
    recobe_chain     *chain;
    recobe_conf       dummy;
    u_int32_t         loop;                          /*            */
    u_int32_t         looe;                          /*             */
    u_int16_t         loovel;                        /*   loop                     */
    u_int8_t          shift;                         /* C                   */
    u_int8_t          dummy0;
#if BLINAP > 4
    int32_t           dummy1;
#endif
    u_int64_t         dummy3;
    recobe_conf       cf;
#  ifndef  _BABOLO_PARSER_H_
    void             *bos;
#  else
    babolo_opts      *bos;
#  endif
};

extern pgoblin_styles PGOBLIN_MODULE;

char   *style_json_geterm __P((style_json_term *term, char *tin));
int     style_json_flags  __P((pgoblin_realsty *st, char **dostyle));
void    style_json_dump   __P((pgoblin_rst *rst, FILE *ouf));
int     style_json_parser __P((pgoblin_rst *rst, char *srcstyle));
ssize_t style_json_table  __P((pgoblin_exenv *exenv, pgoblin_nr *rn, ssize_t low, ssize_t hig));
__END_DECLS

#endif /* !_PGOBLIN_5_JSON_H_ */
